from __future__ import annotations

from typing import Any, Dict, List, NamedTuple, Tuple, Type, Union

from inui.config.replaces import replace


class Attributes:
    def __init__(self, **kwargs: Any) -> None:
        self.attrs: Dict[str, Any] = {key: value for key, value in kwargs.items()}

    # def __getattribute__(self, __name: str) -> Any:
    #     if __name == "attrs":
    #         return super().__getattribute__("attrs")
    #     return self.attrs.get(__name, None)

    def __str__(self) -> str:
        return " ".join(
            f'{replace(name)}="{self._str(value)}"'
            for name, value in self.attrs.items()
            if value is not None
        )

    def fix_values(self, value: str | list = ""):
        if isinstance(value, str):
            return value

    def __getitem__(self, index: str) -> Any:
        return self.attrs.get(index, None)

    def __setitem__(self, index: str, value: Any) -> None:
        self.attrs[index] = value

    def __delitem__(self, index: str) -> None:
        del self.attrs[index]

    def __iter__(self) -> Any:
        return iter(self.attrs)

    def __len__(self) -> int:
        return len(self.attrs)

    def keys(self) -> List[str]:
        return list(self.attrs.keys())

    def values(self) -> List[Any]:
        return list(self.attrs.values())

    def items(self) -> List[Tuple[str, Any]]:
        return list(self.attrs.items())

    def get(self, index: str, default: Any = None) -> Any:
        return self.attrs.get(index, default)

    def update(self, **kwargs: Any) -> None:
        self.attrs.update(kwargs)

    def clear(self) -> None:
        self.attrs.clear()

    def pop(self, index: str, default: Any = None) -> Any:
        return self.attrs.pop(index, default)

    def popitem(self) -> Tuple[str, Any]:
        return self.attrs.popitem()

    def append(self, key: str = "class", *values):
        if key not in self.attrs.keys():
            self.attrs[key] = values
        else:
            if isinstance(self.attrs[key], (list, tuple, set)):
                self.attrs[key] = (*self.attrs[key], *values)
            else:
                self.attrs[key] = (self.attrs[key], *values)

    def _str(self, data, seperator=" ", fill=""):
        try:
            if isinstance(data, type):
                if isinstance(data(), Base):
                    data = data()
        except Exception as _:
            pass

        if isinstance(data, (set, tuple, list)):
            result = seperator.join([str(i) for i in data])
            return f"{fill}{result}{fill}"
        return str(data)

    def copy(self):
        return self.attrs.copy()


class _Data:
    def __init__(self, *args) -> None:
        self._data: List[Any] = args

    def __str__(self) -> str:
        return "".join(map(self._str, self._data))

    def _str(self, data):
        try:
            if isinstance(data, type):
                if isinstance(data(), Base):
                    data = data()
        except Exception as _:
            pass
        return str(data)

    def append(self, *args):
        self._data = (*self._data, *args)

    def copy(self):
        return self._data.copy()


class Meta(NamedTuple):
    start_tag: str = None
    end_tag: str = None
    start_sign: str = "<"
    end_sign: str = ">"
    close_sign: str = "</"
    void_sign: str = "/>"


empty = Meta("", "", "", "", "", "")


class Base:
    Meta: Type[Meta] = Meta()
    _data = _Data()
    _attributes = Attributes()

    def __init__(
        self, *data: Any, _meta: Type[Meta] | None = None, **attributes: Any
    ) -> None:
        self.data = data
        self.attributes = attributes
        self.Meta = _meta or self.Meta

    @property
    def data(self) -> list:
        return self._data

    @data.setter
    def data(self, value: Any) -> None:
        self._data = _Data(*value)

    @property
    def start_tag(self) -> str:
        return (
            self.Meta.start_tag
            if self.Meta.start_tag is not None
            else self.__class__.__name__.lower()
        )

    @start_tag.setter
    def start_tag(self, value: str) -> None:
        self.Meta.start_tag = value

    @property
    def end_tag(self) -> str:
        return (
            self.Meta.end_tag
            if self.Meta.end_tag is not None
            else self.__class__.__name__.lower()
        )

    @end_tag.setter
    def end_tag(self, value: str) -> None:
        self.Meta.end_tag = value

    @property
    def attributes(self) -> Attributes:
        return self._attributes

    @attributes.setter
    def attributes(self, value: Union[Attributes, Dict[str, Any]]) -> None:
        if not isinstance(value, (Attributes, dict)):
            raise TypeError("attributes must be an instance of Attributes or dict")
        self._attributes = (
            Attributes(**value) if isinstance(value, dict) else value.copy()
        )

    def render(self, prettify: bool = False) -> str:

        return f"{self.Meta.start_sign}{self.start_tag}{' ' if len(self.attributes) > 0 else ''}\
{str(self.attributes)}{self.Meta.end_sign}{str(self.data)}{self.Meta.close_sign}{self.end_tag}{self.Meta.end_sign}"

    def __str__(self) -> str:
        return self.render()

    def __repr__(self) -> str:
        return self.render()

    def __add__(self, other) -> str:
        return Base(self.__str__() + " " + self._str(other), _meta=empty)

    def __radd__(self, other) -> str:
        return self._str(other) + " " + self.__str__()

    def __len__(self):
        return len(self.__str__())

    def __getitem__(self, item):
        return self.__str__()[item]

    def _str(self, data):
        try:
            if isinstance(data, type):
                if isinstance(data(), Base):
                    data = data()
        except Exception as _:
            pass
        return str(data)

    def __lt__(self, other):
        return Base(f"{self._str(other)} > {self.__str__()}", _meta=empty)

    def __gt__(self, other):
        return Base(f"{self.__str__()} > {self._str(other)}", _meta=empty)

    def __le__(self, other):
        return Base(f"{self.__str__()} <= {self._str(other)}", _meta=empty)

    def __ge__(self, other):
        return Base(f"{self.__str__()} >= {self._str(other)}", _meta=empty)

    def __mul__(self, value: int) -> str:
        return Base(self.__str__() * int(value), _meta=empty)

    def __rmul__(self, value: int) -> str:
        return self.__mul__(value)

    def __imul__(self, value: int) -> str:
        return self.__mul__(value)

    def css(self, tag: str = ""):
        return tag + self.start_tag

    def __matmul__(self, other):
        return self._temp(other)

        # return self._data.append(other)

    def __and__(self, other):
        return self._temp(other)

    def __or__(self, other):
        return self._temp(other)

    def __mod__(self, other):
        return self._temp(other)

    def __truediv__(self, other):
        if not isinstance(other, (list, set, tuple)):
            other = (other,)
        return self._attributes.append(*other)

    def __imatmul__(self, other):
        return self._itemp(other)

    def __iand__(self, other):
        return self._itemp(other)

    def __ior__(self, other):
        return self._itemp(other)

    def __imod__(self, other):
        return self._itemp(other)

    def __itruediv__(self, other):
        return self._itemp(other)

    def copy(self):
        return self.__class__(*self._data._data, **self._attributes.copy())

    def _temp(self, other):
        if not isinstance(other, (list, set, tuple)):
            other = (other,)
        copy = self.copy()
        copy._attributes.append(
            "class",
            *other,
        )
        return copy

    def _itemp(self, other):
        if not isinstance(other, (list, set, tuple)):
            other = (other,)
        self._attributes.append(
            "class",
            *other,
        )
        return self


class BaseElement(Base): ...


class BaseVoidElement(BaseElement):
    def render(self, prettify: bool = False):
        return f"""{self.Meta.start_sign}{self.start_tag}{' ' if len(self.attributes) > 0 else ''}{str(self.attributes)}{self.Meta.close_sign} """


class BaseSvgElement(Base): ...


class Class(Base):
    def __init__(self, *args) -> None:
        self.args = args

    def render(self, prettify: bool = False) -> str:
        return " ".join(list(map(self._str, self.args)))



class Convert:
    """
    A utility class for handling various unit conversions.

    Conversion Ratios:
    - px: pixels
    - pt: points
    - in: inches
    - cm: centimeters
    - mm: millimeters
    - pc: picas
    - em: em units
    - rem: root em units

    Additional Methods:
    - px_to_percent: Convert pixels to percentage based on a container width.
    - percent_to_px: Convert percentage to pixels based on a container width.
    - cm_to_in: Convert centimeters to inches.
    - in_to_cm: Convert inches to centimeters.
    """

    @staticmethod
    def convert(value: float, ratio: float) -> float:
        return value * ratio

    @staticmethod
    def px_to_pt(px: float) -> float:
        return Convert.convert(px, 0.75)

    @staticmethod
    def pt_to_px(pt: float) -> float:
        return Convert.convert(pt, 1 / 0.75)

    @staticmethod
    def px_to_in(px: float) -> float:
        return Convert.convert(px, 0.0138889)

    @staticmethod
    def in_to_px(inch: float) -> float:
        return Convert.convert(inch, 1 / 0.0138889)

    @staticmethod
    def px_to_cm(px: float) -> float:
        return Convert.convert(px, 0.0352778)

    @staticmethod
    def cm_to_px(cm: float) -> float:
        return Convert.convert(cm, 1 / 0.0352778)

    @staticmethod
    def px_to_mm(px: float) -> float:
        return Convert.convert(px, 0.352778)

    @staticmethod
    def mm_to_px(mm: float) -> float:
        return Convert.convert(mm, 1 / 0.352778)

    @staticmethod
    def px_to_pc(px: float) -> float:
        return Convert.convert(px, 0.05)

    @staticmethod
    def pc_to_px(pc: float) -> float:
        return Convert.convert(pc, 1 / 0.05)

    @staticmethod
    def px_to_em(px: float) -> float:
        return Convert.convert(px, 0.0625)

    @staticmethod
    def em_to_px(em: float) -> float:
        return Convert.convert(em, 1 / 0.0625)

    @staticmethod
    def px_to_rem(px: float) -> float:
        return Convert.convert(px, 0.0625)

    @staticmethod
    def rem_to_px(rem: float) -> float:
        return Convert.convert(rem, 1 / 0.0625)

    @staticmethod
    def px_to_percent(px: float, container_width: float) -> float:
        return (px / container_width) * 100

    @staticmethod
    def percent_to_px(percent: float, container_width: float) -> float:
        return (percent / 100) * container_width

    @staticmethod
    def cm_to_in(cm: float) -> float:
        return cm * 0.393701

    @staticmethod
    def in_to_cm(inch: float) -> float:
        return inch / 0.393701



from typing import List

try:
    from bs4 import BeautifulSoup, Tag
except ImportError:
    raise "you must install bs4, run `python3 -m pip install bs4"


class Pretiffy:
    def __init__(
        self, text: str = "", file_path: str = "", parser: str = "html.parser"
    ):
        """
        Initialize the Pretiffy object.

        Args:
            text (str): The HTML content as a string.
            file_path (str): The path to the HTML file.
            parser (str): The parser to use with BeautifulSoup (default is 'html.parser').
        """
        self.file_path = file_path
        self.text = text
        self.parser = parser
        if self.file_path:
            self.from_file()

    def save(self, out: str = "out.html") -> None:
        """
        Save the prettified HTML content to a file.

        Args:
            out (str): The output file path (default is 'out.html').
        """
        with open(out, "w", encoding="utf-8") as f:
            f.write(self.prettify_string())

    def from_file(self) -> None:
        """
        Read HTML content from a file and set it as the current text.
        """
        if self.file_path:
            with open(self.file_path, "r", encoding="utf-8") as f:
                self.text = f.read()

    def prettify_string(self) -> str:
        """
        Prettify the HTML content using BeautifulSoup.

        Returns:
            str: The prettified HTML content.
        """
        soup = BeautifulSoup(self.text, self.parser)
        return soup.prettify()

    def find_tags(self, tag_name: str) -> List[Tag]:
        """
        Find all occurrences of a specific HTML tag.

        Args:
            tag_name (str): The name of the HTML tag to find.

        Returns:
            List[Tag]: A list containing all occurrences of the specified tag.
        """
        soup = BeautifulSoup(self.text, self.parser)
        return soup.find_all(tag_name)

    def remove_tags(self, tags_to_remove: List[str]) -> str:
        """
        Remove specified HTML tags from the content.

        Args:
            tags_to_remove (List[str]): A list of tag names to remove.

        Returns:
            str: The HTML content after removing the specified tags.
        """
        soup = BeautifulSoup(self.text, self.parser)
        for tag in tags_to_remove:
            for elem in soup.find_all(tag):
                elem.decompose()
        return str(soup)

    def replace_tags(self, old_tag: str, new_tag: str) -> str:
        """
        Replace occurrences of a specific HTML tag with another tag.

        Args:
            old_tag (str): The name of the HTML tag to replace.
            new_tag (str): The name of the HTML tag to replace with.

        Returns:
            str: The HTML content after replacing the specified tags.
        """
        soup = BeautifulSoup(self.text, self.parser)
        for elem in soup.find_all(old_tag):
            elem.name = new_tag
        return str(soup)

    def __str__(self) -> str:
        """
        Return the prettified HTML content as a string.

        Returns:
            str: The prettified HTML content.
        """
        return self.prettify_string()

    def __repr__(self) -> str:
        """
        Return the prettified HTML content as a string.

        Returns:
            str: The prettified HTML content.
        """
        return self.prettify_string()



import argparse
import asyncio
import http.server
import importlib.util
import logging
import os
import socketserver
import threading

try:
    import websockets
except ImportError:
    raise "you must install websockets, run `python3 -m pip install websockets"
try:
    from watchdog.events import FileSystemEventHandler
    from watchdog.observers import Observer
except ImportError:
    raise "you must install watchdog, run `python3 -m pip install watchdog"

file_to_watch = ""
variable_name = ""
clients = set()
sleep_time = 1

parser = argparse.ArgumentParser(description="Hot reload HTML content.")
parser.add_argument("module", type=str, help="Module name in the format main:variable")
parser.add_argument("--host", type=str, default="localhost", help="Host address")
parser.add_argument("--port", type=int, default=8000, help="Port number")
parser.add_argument("--sleep", type=float, default=1, help="Sleep time")
args = parser.parse_args()

module_name, variable_name = args.module.split(":")
file_to_watch = f"{module_name.replace('.', '/')}.py"
sleep_time = args.sleep
logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s"
)


def get_file_contents(file_name, variable_name):
    try:
        module_name = os.path.splitext(os.path.basename(file_name))[0]
        spec = importlib.util.spec_from_file_location(module_name, file_name)
        module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(module)
        return getattr(module, variable_name)
    except Exception as err:
        logging.error(f"Error getting file contents: {err}")
        return f"<h1>{err}</h1>"


def hot_reload_script(host, port):
    return f"""
        <script>
            var ws = new WebSocket("ws://{host}:{port}/ws");
            ws.onopen = function() {{
                console.log("Connected to WebSocket");
            }};
            ws.onmessage = function(event) {{
                console.log("Message received:", event.data);
                document.body.innerHTML = event.data;
            }};
            ws.onclose = function() {{
                console.log("Disconnected from WebSocket");
            }};
        </script>
    """


class HTMLFileHandler(FileSystemEventHandler):
    def on_modified(self, event):
        if event.src_path.endswith(".py"):
            logging.info(
                f"Detected changes in {event.src_path}, updating HTML content."
            )
            asyncio.run(update_html_content(event.src_path))


async def send_update(html_content):
    if clients:
        await asyncio.gather(*(client.send(html_content) for client in clients))
        logging.info("Sent updated HTML content to clients.")


async def websocket_handler(websocket, path):
    clients.add(websocket)
    logging.info("New client connected.")
    try:
        async for _ in websocket:
            pass
    finally:
        clients.remove(websocket)
        logging.info("Client disconnected.")


async def update_html_content(*args):
    html_content = get_file_contents(file_to_watch, variable_name)
    if not isinstance(html_content, str):
        html_content = str(html_content)
    await send_update(html_content)


def monitor_file(file_to_watch: str, sleep_time: float):
    event_handler = HTMLFileHandler()
    observer = Observer()
    observer.schedule(event_handler, path=os.getcwd(), recursive=False)
    observer.start()
    logging.info("Started monitoring file changes.")
    try:
        while True:
            asyncio.run(asyncio.sleep(sleep_time))
    except KeyboardInterrupt:
        observer.stop()
    observer.join()


class RequestHandler(http.server.SimpleHTTPRequestHandler):
    def do_GET(self):
        if self.path == "/":
            content = get_file_contents(
                file_to_watch, variable_name
            ) + hot_reload_script(host=args.host, port=args.port + 1)
            self.send_response(200)
            self.send_header("Content-type", "text/html")
            self.end_headers()
            self.wfile.write(content.encode())
            logging.info("Served HTML content.")


def run_server(host, port):
    with socketserver.TCPServer((host, port), RequestHandler) as httpd:
        logging.info(f"Server started on: http://{host}:{port}")
        httpd.serve_forever()


def main():
    threading.Thread(
        target=monitor_file, args=(file_to_watch, sleep_time), daemon=True
    ).start()
    threading.Thread(
        target=run_server, args=(args.host, args.port), daemon=True
    ).start()

    start_server = websockets.serve(websocket_handler, args.host, args.port + 1)
    asyncio.get_event_loop().run_until_complete(start_server)
    asyncio.get_event_loop().run_forever()


if __name__ == "__main__":
    main()



from typing import Any, Optional

from inui.base import Base, BaseVoidElement
from inui.tailwind.utils import tw_class_to_lower


class BaseCSSValue(BaseVoidElement):
    tag: Optional[str] = None

    def __init__(self, *values) -> None:
        self.values = values

    def render(self, prettify: bool = False) -> str:
        values = ""
        if self.values:
            values += " -"
            values += "-".join(list(map(str, self.values)))
        return self.tag or tw_class_to_lower(self.__class__.__name__) + values


class BaseCSSProperty(BaseVoidElement):
    tag: Optional[str] = None
    type_: Optional[str] = None

    def __init__(self, *values, type_: str = "") -> None:
        self.values = values
        self.type_ = type_

    def render(self, prettify: bool = False) -> str:
        values = ""
        if self.values:
            values += " -"
            values += "-".join(list(map(str, self.values)))
        return self.type_ + (
            self.tag or tw_class_to_lower(self.__class__.__name__) + values
        )


class CSS(Base):
    def __init__(self, *args, **attributes: Any) -> None:
        self.attrs = attributes
        self.args = args

    def render(self, inline: bool = True, prettify: bool = False) -> str:
        if inline:
            return f'"{self._css()}"'
        return "{%s}" % self._css()

    def _css(self):
        result = ""
        for i in self.args:
            if isinstance(i, dict):
                for key, value in i.items():
                    result += f"{self._str(key)}: {self._str(value)};"
        for key, value in self.attrs.items():
            result += f"{self._str(key)}: {self._str(value)};"
        return result


class align_content(BaseCSSProperty):
    class stretch(BaseCSSValue): ...

    class center(BaseCSSValue): ...

    class flex_start(BaseCSSValue): ...

    class flex_end(BaseCSSValue): ...

    class space_between(BaseCSSValue): ...

    class space_around(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...


class align_items(BaseCSSProperty):
    class stretch(BaseCSSValue): ...

    class center(BaseCSSValue): ...

    class flex_start(BaseCSSValue): ...

    class flex_end(BaseCSSValue): ...

    class baseline(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...


class align_self(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class stretch(BaseCSSValue): ...

    class center(BaseCSSValue): ...

    class flex_start(BaseCSSValue): ...

    class flex_end(BaseCSSValue): ...

    class baseline(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...


class all(BaseCSSProperty):
    class unset(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class revert(BaseCSSValue): ...

    class auto(BaseCSSValue): ...

    class none(BaseCSSValue): ...


class animation(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class animation_delay(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class animation_direction(BaseCSSProperty):
    class normal(BaseCSSValue): ...

    class reverse(BaseCSSValue): ...

    class alternate(BaseCSSValue): ...

    class alternate_reverse(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class animation_duration(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class animation_fill_mode(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class forwards(BaseCSSValue): ...

    class backwards(BaseCSSValue): ...

    class both(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class animation_iter_count(BaseCSSProperty):
    class infinite(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class animation_name(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class animation_play_state(BaseCSSProperty):
    class running(BaseCSSValue): ...

    class paused(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class animation_timing_fn(BaseCSSProperty):
    class ease(BaseCSSValue): ...

    class ease_in(BaseCSSValue): ...

    class ease_out(BaseCSSValue): ...

    class ease_in_out(BaseCSSValue): ...

    class linear(BaseCSSValue): ...

    class step_start(BaseCSSValue): ...

    class step_end(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class backface_visibility(BaseCSSProperty):
    class visible(BaseCSSValue): ...

    class hidden(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class background(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class background_attachment(BaseCSSProperty):
    class scroll(BaseCSSValue): ...

    class fixed(BaseCSSValue): ...

    class local(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class background_blend_mode(BaseCSSProperty):
    class normal(BaseCSSValue): ...

    class multiply(BaseCSSValue): ...

    class screen(BaseCSSValue): ...

    class overlay(BaseCSSValue): ...

    class darken(BaseCSSValue): ...

    class lighten(BaseCSSValue): ...

    class color_dodge(BaseCSSValue): ...

    class color_burn(BaseCSSValue): ...

    class hard_light(BaseCSSValue): ...

    class soft_light(BaseCSSValue): ...

    class difference(BaseCSSValue): ...

    class exclusion(BaseCSSValue): ...

    class hue(BaseCSSValue): ...

    class saturation(BaseCSSValue): ...

    class color(BaseCSSValue): ...

    class luminosity(BaseCSSValue): ...


class background_clip(BaseCSSProperty):
    class border_box(BaseCSSValue): ...

    class padding_box(BaseCSSValue): ...

    class content_box(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class background_color(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class background_image(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class background_origin(BaseCSSProperty):
    class border_box(BaseCSSValue): ...

    class padding_box(BaseCSSValue): ...

    class content_box(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class background_position(BaseCSSProperty):
    class top_left(BaseCSSValue): ...

    class top_right(BaseCSSValue): ...

    class bottom_left(BaseCSSValue): ...

    class bottom_right(BaseCSSValue): ...

    class center(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class background_repeat(BaseCSSProperty):
    class repeat(BaseCSSValue): ...

    class repeat_x(BaseCSSValue): ...

    class repeat_y(BaseCSSValue): ...

    class no_repeat(BaseCSSValue): ...

    class space(BaseCSSValue): ...

    class round(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class background_size(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class cover(BaseCSSValue): ...

    class contain(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_bottom(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_bottom_color(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_bottom_left_rad(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_bottom_right_ra(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_bottom_style(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class solid(BaseCSSValue): ...

    class dashed(BaseCSSValue): ...

    class dotted(BaseCSSValue): ...

    class double(BaseCSSValue): ...

    class groove(BaseCSSValue): ...

    class ridge(BaseCSSValue): ...

    class inset(BaseCSSValue): ...

    class outset(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_bottom_width(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_collapse(BaseCSSProperty):
    class separate(BaseCSSValue): ...

    class collapse(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_color(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_image(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_image_outset(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_image_repeat(BaseCSSProperty):
    class stretch(BaseCSSValue): ...

    class repeat(BaseCSSValue): ...

    class round(BaseCSSValue): ...

    class space(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_image_slice(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_image_source(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_image_width(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_left(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_left_color(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_left_style(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class solid(BaseCSSValue): ...

    class dashed(BaseCSSValue): ...

    class dotted(BaseCSSValue): ...

    class double(BaseCSSValue): ...

    class groove(BaseCSSValue): ...

    class ridge(BaseCSSValue): ...

    class inset(BaseCSSValue): ...

    class outset(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_left_width(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_radius(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_right(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_right_color(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_right_style(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class solid(BaseCSSValue): ...

    class dashed(BaseCSSValue): ...

    class dotted(BaseCSSValue): ...

    class double(BaseCSSValue): ...

    class groove(BaseCSSValue): ...

    class ridge(BaseCSSValue): ...

    class inset(BaseCSSValue): ...

    class outset(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_right_width(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_spacing(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_style(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class solid(BaseCSSValue): ...

    class dashed(BaseCSSValue): ...

    class dotted(BaseCSSValue): ...

    class double(BaseCSSValue): ...

    class groove(BaseCSSValue): ...

    class ridge(BaseCSSValue): ...

    class inset(BaseCSSValue): ...

    class outset(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_top(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_top_color(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_top_left_radius(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_top_right_radiu(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_top_style(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class solid(BaseCSSValue): ...

    class dashed(BaseCSSValue): ...

    class dotted(BaseCSSValue): ...

    class double(BaseCSSValue): ...

    class groove(BaseCSSValue): ...

    class ridge(BaseCSSValue): ...

    class inset(BaseCSSValue): ...

    class outset(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_top_width(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class border_width(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class bottom(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class box_decoration_break(BaseCSSProperty):
    class slice(BaseCSSValue): ...

    class clone(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class box_shadow(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class box_sizing(BaseCSSProperty):
    class content_box(BaseCSSValue): ...

    class border_box(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class caption_side(BaseCSSProperty):
    class top(BaseCSSValue): ...

    class bottom(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class caret_color(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class charset(BaseCSSProperty):
    tag = "@charset"


class clear(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class left(BaseCSSValue): ...

    class right(BaseCSSValue): ...

    class both(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class clip(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class clip_path(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class color(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class column_count(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class column_fill(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class balance(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class column_gap(BaseCSSProperty):
    class normal(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class column_rule(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class column_rule_color(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class column_rule_style(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class solid(BaseCSSValue): ...

    class dashed(BaseCSSValue): ...

    class dotted(BaseCSSValue): ...

    class double(BaseCSSValue): ...

    class groove(BaseCSSValue): ...

    class ridge(BaseCSSValue): ...

    class inset(BaseCSSValue): ...

    class outset(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class column_rule_width(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class column_span(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class all(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class column_width(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class columns(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class content(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class counter_increment(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class counter_reset(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class cursor(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class pointer(BaseCSSValue): ...

    class crosshair(BaseCSSValue): ...

    class text(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class direction(BaseCSSProperty):
    class ltr(BaseCSSValue): ...

    class rtl(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class display(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class inline(BaseCSSValue): ...

    class block(BaseCSSValue): ...

    class flex(BaseCSSValue): ...

    class grid(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class empty_cells(BaseCSSProperty):
    class show(BaseCSSValue): ...

    class hide(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class filter(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class flex(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class flex_basis(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class flex_direction(BaseCSSProperty):
    class row(BaseCSSValue): ...

    class row_reverse(BaseCSSValue): ...

    class column(BaseCSSValue): ...

    class column_reverse(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class flex_flow(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class flex_grow(BaseCSSProperty):

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class flex_shrink(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class flex_wrap(BaseCSSProperty):
    class nowrap(BaseCSSValue): ...

    class wrap(BaseCSSValue): ...

    class wrap_reverse(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class float(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class left(BaseCSSValue): ...

    class right(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class font(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class font_face(BaseCSSProperty):
    tag = "@font-face"


class font_family(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class font_kerning(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class none(BaseCSSValue): ...

    class normal(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class font_size(BaseCSSProperty):
    class xx_small(BaseCSSValue): ...

    class x_small(BaseCSSValue): ...

    class small(BaseCSSValue): ...

    class medium(BaseCSSValue): ...

    class large(BaseCSSValue): ...

    class x_large(BaseCSSValue): ...

    class xx_large(BaseCSSValue): ...

    class larger(BaseCSSValue): ...

    class smaller(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class font_stretch(BaseCSSProperty):
    class ultra_condensed(BaseCSSValue): ...

    class extra_condensed(BaseCSSValue): ...

    class condensed(BaseCSSValue): ...

    class semi_condensed(BaseCSSValue): ...

    class normal(BaseCSSValue): ...

    class semi_expanded(BaseCSSValue): ...

    class expanded(BaseCSSValue): ...

    class extra_expanded(BaseCSSValue): ...

    class ultra_expanded(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class font_style(BaseCSSProperty):
    class normal(BaseCSSValue): ...

    class italic(BaseCSSValue): ...

    class oblique(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class font_variant(BaseCSSProperty):
    class normal(BaseCSSValue): ...

    class small_caps(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class font_weight(BaseCSSProperty):
    class normal(BaseCSSValue): ...

    class bold(BaseCSSValue): ...

    class bolder(BaseCSSValue): ...

    class lighter(BaseCSSValue): ...

    class weight_100(BaseCSSValue): ...

    class weight_200(BaseCSSValue): ...

    class weight_300(BaseCSSValue): ...

    class weight_400(BaseCSSValue): ...

    class weight_500(BaseCSSValue): ...

    class weight_600(BaseCSSValue): ...

    class weight_700(BaseCSSValue): ...

    class weight_800(BaseCSSValue): ...

    class weight_900(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class gap(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class grid(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class grid_area(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class span(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class grid_auto_columns(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class max_content(BaseCSSValue): ...

    class min_content(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class grid_auto_flow(BaseCSSProperty):
    class row(BaseCSSValue): ...

    class column(BaseCSSValue): ...

    class dense(BaseCSSValue): ...

    class row_dense(BaseCSSValue): ...

    class column_dense(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class grid_auto_rows(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class max_content(BaseCSSValue): ...

    class min_content(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class grid_column(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class span(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class grid_column_end(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class span(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class grid_column_gap(BaseCSSProperty):
    class normal(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class grid_column_start(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class span(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class grid_gap(BaseCSSProperty):
    class normal(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class grid_row(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class span(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class grid_row_end(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class span(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class grid_row_gap(BaseCSSProperty):
    class normal(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class grid_row_start(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class span(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class grid_template(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class grid_template_areas(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class grid_template_columns(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class max_content(BaseCSSValue): ...

    class min_content(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class grid_template_rows(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class max_content(BaseCSSValue): ...

    class min_content(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class height(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class hyphens(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class manual(BaseCSSValue): ...

    class auto(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class import_(BaseCSSProperty):
    tag = "@import"


class justify_content(BaseCSSProperty):
    class flex_start(BaseCSSValue): ...

    class flex_end(BaseCSSValue): ...

    class center(BaseCSSValue): ...

    class space_between(BaseCSSValue): ...

    class space_around(BaseCSSValue): ...

    class space_evenly(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class keyframes(BaseCSSProperty):
    tag = "@keyframes"


class left(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class letter_spacing(BaseCSSProperty):
    class normal(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class line_height(BaseCSSProperty):
    class normal(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class list_style(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class list_style_image(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class list_style_position(BaseCSSProperty):
    class inside(BaseCSSValue): ...

    class outside(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class list_style_type(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class disc(BaseCSSValue): ...

    class circle(BaseCSSValue): ...

    class square(BaseCSSValue): ...

    class decimal(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class margin(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class margin_bottom(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class margin_left(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class margin_right(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class margin_top(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class max_height(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class max_width(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class media(BaseCSSProperty):
    tag = "@media"


class min_height(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class min_width(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class object_fit(BaseCSSProperty):
    class fill(BaseCSSValue): ...

    class contain(BaseCSSValue): ...

    class cover(BaseCSSValue): ...

    class none(BaseCSSValue): ...

    class scale_down(BaseCSSValue): ...


class object_position(BaseCSSProperty):
    class top(BaseCSSValue): ...

    class bottom(BaseCSSValue): ...

    class left(BaseCSSValue): ...

    class right(BaseCSSValue): ...

    class center(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class opacity(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class order(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class outline(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class outline_color(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...

    class transparent(BaseCSSValue): ...


class outline_offset(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class outline_style(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class dotted(BaseCSSValue): ...

    class dashed(BaseCSSValue): ...

    class solid(BaseCSSValue): ...

    class double(BaseCSSValue): ...

    class groove(BaseCSSValue): ...

    class ridge(BaseCSSValue): ...

    class inset(BaseCSSValue): ...

    class outset(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class outline_width(BaseCSSProperty):
    class thin(BaseCSSValue): ...

    class medium(BaseCSSValue): ...

    class thick(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class overflow(BaseCSSProperty):
    class visible(BaseCSSValue): ...

    class hidden(BaseCSSValue): ...

    class scroll(BaseCSSValue): ...

    class auto(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class overflow_x(BaseCSSProperty):
    class visible(BaseCSSValue): ...

    class hidden(BaseCSSValue): ...

    class scroll(BaseCSSValue): ...

    class auto(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class overflow_y(BaseCSSProperty):
    class visible(BaseCSSValue): ...

    class hidden(BaseCSSValue): ...

    class scroll(BaseCSSValue): ...

    class auto(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class padding(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class padding_bottom(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class padding_left(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class padding_right(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class padding_top(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class page_break_after(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class always(BaseCSSValue): ...

    class avoid(BaseCSSValue): ...

    class left(BaseCSSValue): ...

    class right(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class page_break_before(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class always(BaseCSSValue): ...

    class avoid(BaseCSSValue): ...

    class left(BaseCSSValue): ...

    class right(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class page_break_inside(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class avoid(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class perspective(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class perspective_origin(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class pointer_events(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class none(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class position(BaseCSSProperty):
    class static(BaseCSSValue): ...

    class relative(BaseCSSValue): ...

    class absolute(BaseCSSValue): ...

    class fixed(BaseCSSValue): ...

    class sticky(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class quotes(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class right(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class row_gap(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class scroll_behavior(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class smooth(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class table_layout(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class fixed(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class text_align(BaseCSSProperty):
    class left(BaseCSSValue): ...

    class right(BaseCSSValue): ...

    class center(BaseCSSValue): ...

    class justify(BaseCSSValue): ...

    class start(BaseCSSValue): ...

    class end(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class text_align_last(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class left(BaseCSSValue): ...

    class right(BaseCSSValue): ...

    class center(BaseCSSValue): ...

    class justify(BaseCSSValue): ...

    class start(BaseCSSValue): ...

    class end(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class text_decoration(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class underline(BaseCSSValue): ...

    class overline(BaseCSSValue): ...

    class line_through(BaseCSSValue): ...

    class blink(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class text_decoration_color(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class text_decoration_line(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class underline(BaseCSSValue): ...

    class overline(BaseCSSValue): ...

    class line_through(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class text_decoration_style(BaseCSSProperty):
    class solid(BaseCSSValue): ...

    class double(BaseCSSValue): ...

    class dotted(BaseCSSValue): ...

    class dashed(BaseCSSValue): ...

    class wavy(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class text_indent(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class text_justify(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class none(BaseCSSValue): ...

    class inter_word(BaseCSSValue): ...

    class distribute(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class text_overflow(BaseCSSProperty):
    class clip(BaseCSSValue): ...

    class ellipsis(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class text_shadow(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class text_transform(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class capitalize(BaseCSSValue): ...

    class uppercase(BaseCSSValue): ...

    class lowercase(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class top(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class transform(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class transform_origin(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class transform_style(BaseCSSProperty):
    class flat(BaseCSSValue): ...

    class preserve_3d(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class transition(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class transition_delay(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class transition_duration(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class transition_property(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class all(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class transition_timing_fn(BaseCSSProperty):
    class ease(BaseCSSValue): ...

    class linear(BaseCSSValue): ...

    class ease_in(BaseCSSValue): ...

    class ease_out(BaseCSSValue): ...

    class ease_in_out(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class user_select(BaseCSSProperty):
    class none(BaseCSSValue): ...

    class text(BaseCSSValue): ...

    class all(BaseCSSValue): ...

    class auto(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class vertical_align(BaseCSSProperty):
    class baseline(BaseCSSValue): ...

    class sub(BaseCSSValue): ...

    class super(BaseCSSValue): ...

    class text_top(BaseCSSValue): ...

    class text_bottom(BaseCSSValue): ...

    class middle(BaseCSSValue): ...

    class top(BaseCSSValue): ...

    class bottom(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class visibility(BaseCSSProperty):
    class visible(BaseCSSValue): ...

    class hidden(BaseCSSValue): ...

    class collapse(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class white_space(BaseCSSProperty):
    class normal(BaseCSSValue): ...

    class nowrap(BaseCSSValue): ...

    class pre(BaseCSSValue): ...

    class pre_wrap(BaseCSSValue): ...

    class pre_line(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class width(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class word_break(BaseCSSProperty):
    class normal(BaseCSSValue): ...

    class break_all(BaseCSSValue): ...

    class keep_all(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class word_spacing(BaseCSSProperty):
    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class word_wrap(BaseCSSProperty):
    class normal(BaseCSSValue): ...

    class break_word(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class writing_mode(BaseCSSProperty):
    class horizontal_tb(BaseCSSValue): ...

    class vertical_rl(BaseCSSValue): ...

    class vertical_lr(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...


class z_index(BaseCSSProperty):
    class auto(BaseCSSValue): ...

    class inherit(BaseCSSValue): ...

    class initial(BaseCSSValue): ...

    class unset(BaseCSSValue): ...



from typing import Optional

from inui.base import Base


class Size(Base):
    metric: Optional[str] = None

    def __init__(self, value=0) -> None:
        self.value = value

    def render(self, prettify: bool = False) -> str:
        metric = self.metric or self.__class__.__name__.lower()
        return f"{self.value}{metric}"


class Px(Size): ...


class Rem(Size): ...


class Em(Size): ...


class Percent(Size):
    metric = "%"


class VW(Size): ...


class VH(Size): ...


class VMIN(Size): ...


class VMin(Size): ...


class VMax(Size): ...


class EX(Size): ...


class CH(Size): ...


class PT(Size): ...


class CM(Size): ...


class PC(Size): ...


class MM(Size): ...


class IN(Size): ...



replaceDict = {
    "classs": "class",
    "class_": "class",
    "forr": "for",
    "for_": "for",
    "accessskey": "accesskey",
    "access_key": "accesskey",
    "aria_lable": "aria-label",
    "arialable": "aria-label",
    "aria_describedby": "aria-describedby",
    "ariadescribedby": "aria-describedby",
    "iss": "is",
    "is_": "is",
    "doctype": "!DOCTYPE HTML",
    "typee": "type",
    "type_": "type",
    "aria_expanded": "aria-expanded",
    "ariaexpanded": "aria-expanded",
    "asyncc": "async",
    "async_": "async",
    "data_expandable": "data-expandable",
    "dataexpandable": "data-expandable",
    "data_parent": "data-parent",
    "dataparent": "data-parent",
    "data_child": "data-child",
    "datachild": "data-child",
    "data_nl": "data-nl",
    "datanl": "data-nl",
    "data_sm": "data-sm",
    "datasm": "data-sm",
    "data_mode": "data-mode",
    "datamode": "data-mode",
    "data_gfg_action": "data-gfg-action",
    "data_expanded": "data-expanded",
    "data_show": "data-show",
    "data_article_slider": "data-article-slider",
    "data_bookmark_value": "data-bookmark-value",
    "data_type": "data-type",
    "aria_hidden": "aria-hidden",
    "data_icon": "data-icon",
    "data_rating": "data-rating",
    "datagfg_action": "data-gfg-action",
    "dataexpanded": "data-expanded",
    "datashow": "data-show",
    "dataarticle_slider": "data-article-slider",
    "databookmark_value": "data-bookmark-value",
    "datatype": "data-type",
    "ariahidden": "aria-hidden",
    "dataicon": "data-icon",
    "datarating": "data-rating",
    "fromm": "from",
    "from_": "from",
    "inn": "in",
    "in_": "in",
}


def replace(string: str):
    if string in replaceDict:
        return replaceDict[string]
    string = string.replace("_", "-", string.count("-"))
    string = string.replace("___", " ", string.count(" "))
    string = string.replace("__", ":", string.count(":"))
    string = string.replace("_____", "*", string.count("*"))
    return string


def reverse_replace(string: str):
    for i in replaceDict:
        if string == replaceDict[i]:
            return i
    string = string.replace("-", "_", string.count("-"))
    string = string.replace(" ", "___", string.count(" "))
    string = string.replace(":", "__", string.count(":"))
    string = string.replace("*", "_____", string.count("*"))
    return string
